@@ -13,31 +13,30 @@ for (let i = 0; i < chars.length; i++) {
 }
 
 export function decode(mappings: string): SourceMapMappings {
-	let generatedCodeColumn = 0; // first field
-	let sourceFileIndex = 0;     // second field
-	let sourceCodeLine = 0;      // third field
-	let sourceCodeColumn = 0;    // fourth field
-	let nameIndex = 0;           // fifth field
-
 	const decoded: SourceMapMappings = [];
 	let line: SourceMapLine = [];
-	let segment: number[] = [];
+	const segment: SourceMapSegment = [
+		0, // generated code column
+		0, // source file index
+		0, // source code line
+		0, // source code column
+		0, // name index
+	];
 
-	for (let i = 0, j = 0, shift = 0, value = 0, len = mappings.length; i < len; i++) {
+	let j = 0;
+	for (let i = 0, shift = 0, value = 0; i < mappings.length; i++) {
 		const c = mappings.charCodeAt(i);
 
 		if (c === 44) { // ","
-			if (segment.length) line.push(new Int32Array(segment) as any);
-			segment = [];
+			segmentify(line, segment, j);
 			j = 0;
 
 		} else if (c === 59) { // ";"
-			if (segment.length) line.push(new Int32Array(segment) as any);
-			segment = [];
+			segmentify(line, segment, j);
 			j = 0;
 			decoded.push(line);
 			line = [];
-			generatedCodeColumn = 0;
+			segment[0] = 0;
 
 		} else {
 			let integer = charToInteger[c];
@@ -54,43 +53,38 @@ export function decode(mappings: string): SourceMapMappings {
 				shift += 5;
 			} else {
 				const shouldNegate = value & 1;
-				value >>= 1;
+				value >>>= 1;
 
-				const num = shouldNegate ? -value : value;
-
-				if (j == 0) {
-					generatedCodeColumn += num;
-					segment.push(generatedCodeColumn);
-
-				} else if (j === 1) {
-					sourceFileIndex += num;
-					segment.push(sourceFileIndex);
-
-				} else if (j === 2) {
-					sourceCodeLine += num;
-					segment.push(sourceCodeLine);
-
-				} else if (j === 3) {
-					sourceCodeColumn += num;
-					segment.push(sourceCodeColumn);
-
-				} else if (j === 4) {
-					nameIndex += num;
-					segment.push(nameIndex);
+				if (shouldNegate) {
+					value = value === 0 ? -0x80000000 : -value;
 				}
 
+				segment[j] += value;
 				j++;
 				value = shift = 0; // reset
 			}
 		}
 	}
 
-	if (segment.length) line.push(new Int32Array(segment) as any);
+	segmentify(line, segment, j);
 	decoded.push(line);
 
 	return decoded;
 }
 
+function segmentify(line: SourceMapSegment[], segment: SourceMapSegment, j: number) {
+	// This looks ugly, but we're creating specialized arrays with a specific
+	// length. This is much faster than creating a new array (which v8 expands to
+	// a capacity of 17 after pushing the first item), or slicing out a subarray
+	// (which is slow). Length 4 is assumed to be the most frequent, followed by
+	// length 5 (since not everything will have an associated name), followed by
+	// length 1 (it's probably rare for a source substring to not have an
+	// associated segment data).
+	if (j === 4) line.push([segment[0], segment[1], segment[2], segment[3]]);
+	else if (j === 5) line.push([segment[0], segment[1], segment[2], segment[3], segment[4]]);
+	else if (j === 1) line.push([segment[0]]);
+}
+
 export function encode(decoded: SourceMapMappings): string {
 	let sourceFileIndex = 0;  // second field
 	let sourceCodeLine = 0;   // third field
@@ -141,7 +135,7 @@ function encodeInteger(num: number): string {
 	num = num < 0 ? (-num << 1) | 1 : num << 1;
 	do {
 		var clamped = num & 31;
-		num >>= 5;
+		num >>>= 5;
 		if (num > 0) {
 			clamped |= 32;
 		}
